<?php

namespace App\Console\Commands;

use Barryvdh\DomPDF\Facade as PDF;
use Illuminate\Console\Command;
use Illuminate\Support\Arr;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;
use Intervention\Image\Facades\Image;

class AuctionGiftCards extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'auction:gift-cards
                                {value? : The value of generated gift card}
                                {number? : The number of generated gift cards}
                                {key-length? : The length of the generated gift card unique key}
                                ';

    /**
     * The console command examples.
     *
     *  php artisan auction:gift-cards value=200 number=10 key-length=12
     *  php artisan auction:gift-cards value=200 number=10
     *  php artisan auction:gift-cards value=200
     *  php artisan auction:gift-cards
     *
     */



    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Generate promotion Gift Cards for new users';

    public $defaults = [];

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->defaults = [
            'value' => 100,
            'number' => 1,
            'key-length' => 8,
            'run_time' => Carbon::now()->format('YmdHs'),
            'cycle_limit' => 5000,
            'img_paths' => []
        ];

        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {

        if(!File::exists(storage_path('app/public/auction/giftcards/'. $this->defaults['run_time'] . '/'))) {
            File::makeDirectory(storage_path('app/public/auction/giftcards/'. $this->defaults['run_time'] . '/'), 0775, true);
        }

        $value = Str::after($this->argument('value'), '=');
        $number = Str::after($this->argument('number'), '=');
        $key_length = Str::after($this->argument('key-length'), '=');

        if(!$value) $value = $this->defaults['value'];
        if(!$number) $number = $this->defaults['number'];
        if(!$key_length) $key_length = $this->defaults['key-length'];

        $this->writeCommandLine(true, 'Starting command: auction:gift-cards with parameters below:');
        $this->writeCommandLine(false, 'value=' . $value);
        $this->writeCommandLine(false, 'number=' . $number);
        $this->writeCommandLine(false, 'key-length=' . $key_length);

        $this->writeLogLine(true, 'Starting command: auction:gift-cards with parameters: 
            value=' . $value . ' | 
            number=' . $number . ' | 
            key-length=' . $key_length);


        $this->writeCommandLine(false, '----------------------------------------------');
        $this->writeLogLine(false, '----------------------------------------------');

        $watch_dog = $this->defaults['cycle_limit'];
        $y = 0;
        for($i = 0; $i < $number; $i++){

            $generated_key = $this->generateKey($key_length, $this->defaults['cycle_limit'], 0);

            if(!$this->createGiftCard($value, $generated_key)){
                $i--;
            }
            else{
                $act_path = $this->createCardImage($generated_key, '$' . $value, $this->defaults['run_time']);
                $this->defaults['img_paths'] = Arr::add($this->defaults['img_paths'], $i, $act_path);
            }

            $y++;

            if($y >= $watch_dog){
                $this->writeCommandLine(true, 'Gift card generation reached the cycle limit!');
                $this->writeLogLine(true, 'Gift card generation reached the cycle limit!');
                break;
            }
        }

        $this->generatePdf($this->defaults['img_paths'], $this->defaults['run_time']);

        $this->writeCommandLine(false, '----------------------------------------------');
        $this->writeLogLine(false, '----------------------------------------------');
        $this->writeCommandLine(true, 'End of command: auction:gift-cards.');
        $this->writeLogLine(true, 'End of command: auction:gift-cards.');
    }

    public function generatePdf($cards, $ts){

        $this->writeCommandLine(true, 'Generating PDF file...');
        $this->writeLogLine(true, 'Generating PDF file...');

        $html = '<html><head><style>@page { margin: 1cm 1cm }</style></head><body><table cellspacing="1" cellpadding="0"><thead><tr><th></th><th></th></tr></thead><tbody>';

        for($i = 0; $i < count($cards); $i++) {
            if($i % 2 == 0) {
                $html .= '<tr><td><img src="' . Arr::get($cards, $i) . '" height="223" width="370"></td>';
            }
            else {
                $html .= '<td><img src="' . Arr::get($cards, $i) . '" height="223" width="370"></td></tr>';
            }
        }

        $html .= '</tr></tbody></table></body></html>';

        $pdf = PDF::loadHTML($html);
        $pdf_path = storage_path('app/public/auction/giftcards/coupons_' . $ts . '.pdf');
        $pdf->save($pdf_path);

        $this->writeCommandLine(true, 'PDF file created at location : ' . $pdf_path);
        $this->writeLogLine(true, 'PDF file created at location : ' . $pdf_path);
    }


    public function createCardImage($text, $price, $ts){
        $img = Image::make(storage_path('giftcard/giftcard_template.png'));

        $img->text($text, 775, 490, function($font) {
            $font->file(public_path('fonts/roboto/Roboto-Bold.ttf'));
            $font->size(84);
            $font->color('#415796');
            $font->align('center');
            $font->valign('top');
        });

        $img->text($price, 805, 185, function($font) {
            $font->file(public_path('fonts/roboto/Roboto-BoldItalic.ttf'));
            $font->size(165);
            $font->color('#ff4d55');
            $font->align('center');
            $font->valign('top');
        });

        $path = storage_path('app/public/auction/giftcards/'. $ts . '/' . $text . '.png');

        $img->save($path);

        return $path;
    }

    /**
     * Write to the console.
     *
     * @return void
     */
    public function writeCommandLine($with_ts, $output){
        if($with_ts) {
            $this->info(Carbon::now(). ' - ' . $output);
        }
        else{
            $this->info($output);
        }
    }

    /**
     * Write to the log file.
     *
     * @return void
     */
    public function writeLogLine($with_ts, $output){
        if($with_ts) {
            Log::info(Carbon::now(). ' - ' . $output);
        }
        else{
            Log::info($output);
        }
    }

    /**
     * Checks that the generated key is unique in the existing collection.
     *
     * @return bool
     */
    public function isUniqueGiftCardKey($key){
        $exists = \App\Models\GiftCards::where('hash', $key)->get();

        if(count($exists) == 0){
            return true;
        }
        else{
            return false;
        }
    }

    /**
     * Generates key for the new gift card.
     *
     * @return string [with recursion]
     */
    public function generateKey($length, $cycle_limit, $recursion)
    {
        $key = Str::upper(Str::random($length));
        if (!$this->isUniqueGiftCardKey($key)) {
            if ($recursion <= $cycle_limit) {
                $this->generateKey($length, $cycle_limit, $recursion + 1);
            }
            else{
                return 'Key generation recursion limit reached!';
            }
        }
        else {
            return $key;
        }
    }

    /**
     * Generates new gift card models.
     *
     * @return bool
     */
    public function createGiftCard($value, $key){
        if($key != 'Key generation recursion limit reached!') {

            $giftCard = new \App\Models\GiftCards;
            $giftCard->hash = $key;
            $giftCard->amount = $value;
            $giftCard->amount_left = $value;
            $giftCard->deleted = 0;
            $giftCard->owner_id = null;
            $giftCard->title = 'Promotion';
            if ($giftCard->save()) {
                $this->writeCommandLine(false, $key);
                $this->writeLogLine(false, $key);
                return true;
            } else {
                $this->writeCommandLine(false, 'Cannot create with key: ' . $key);
                $this->writeLogLine(false, 'Cannot create with key: ' . $key);
                return false;
            }
        }
        else{
            $this->writeCommandLine(false, 'Cannot create with key: ' . $key);
            $this->writeLogLine(false, 'Cannot create with key: ' . $key);
            return false;
        }
    }
}
