<?php

namespace App\Models;

use App\Facades\NotificationManager;
use Illuminate\Foundation\Auth\User as Authenticatable;
//use Illuminate\Database\Eloquent\SoftDeletes;
use ApcManager, Hash, Config, Session, Date, Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Notifications\Notifiable;
use Spatie\Permission\Traits\HasRoles;
use \Spatie\Permission\Models\Role;

use App\Notifications\CustomResetPassword as ResetPasswordNotification;


class User extends Authenticatable
{
    //use SoftDeletes;
    use Notifiable;
    use HasRoles;

    protected $table = 'users';
    protected $dates = ['created_at', 'updated_at', 'activated_at', 'verified_at', 'suspended_at', 'suspension_end', 'last_login'];
    protected $fillable = ['first_name', 'last_name', 'email', 'password', 'secondary_password', 'username'];
    protected $hidden   = ['password', 'remember_token'];
    protected $hashableAttributes = ['password','secondary_password'];


    public static function getUsersIdsForStatistic(){
        return User::whereNotIn('id', array_merge(User::getAdminIds(), User::getBotIds()))->pluck('id');
    }

    public static function getAdminIds(){
        $users = User::whereHas('roles', function($role) {
            $role->where('name', 'admin')
                ->orWhere('name', 'support');
        })->pluck('id')->toArray();
        return array_unique($users);
    }

    public static function getBotIds()
    {
        return [
            100000001,
            100000002,
            100000003,
            100000004,
            100000005,
            100000006,
            100000007,
            100000008,
            100000009,
            1000000010,
            1000000011,
            1000000012,
            1000000013,
            1000000014,
            1000000015,
            1000000016,
            1000000017,
            1000000018,
            1000000019,
            1000000020,
            1000000021/*,
            1000000022,*/
        ];
    }

    public function setPasswordAttribute( $password )
    {

        //$this->attributes['password'] = Hash::make( $password );
        $this->attributes['password'] = $password ;
    }

    public function setSecondaryPasswordAttribute( $password )
    {
        $this->attributes['secondary_password'] = Hash::make( $password );
    }

    public function checkPassword($password, $which = 'password')
    {
        return Hash::check($password, $this->$which);
    }


    /*
    |--------------------------------------------------------------------------
    | Relationship Methods
    |--------------------------------------------------------------------------
    */

    public function profile()
    {
        return $this->hasOne('App\Models\UserProfile', 'id', 'id');
    }

    public function support()
    {
        return $this->belongsTo('\App\Models\Support', 'user_id');
    }

    public function messages()
    {
        return $this->belongsTo('\App\Models\Messages', 'user_id', 'id');
    }

    public function position() {
        $pos = $this->hasOne('\App\Models\UsersPositions', 'user_id', 'id')
                ->join('positions', 'users_positions.position_id', '=', 'positions.id')->first();

        return $pos;
    }

    public function createActivation( $data = [] ) {
        if( !empty( $data ) ) {
            $ah = new \App\Models\ActivationHistory;
            $ah->user_id = $this->id;
            $ah->type = $data['type'];
            $ah->status = $data['status'];
            $ah->save();
        }

        if(config('mlm.mlm_enabled')) {
            if (empty($data) || $ah->status) {
                $this->sponsor->activeStatus = 1;
                $this->sponsor->lastActivationDate = Carbon::now();
                $this->sponsor->save();
            }
        }
    }

    public function isStatusActive() {
        return ( is_null( $this->sponsor ) ) ? false : $this->sponsor->isStatusActive();
    }

    public function sponsor()
    {
        return $this->hasOne('App\Models\Sponsor', 'user_id');
    }

    public function SupportReplies()
    {
        return $this->belongsTo('App\Models\SupportReplies', 'user_id');
    }

    public function isOnline()
    {
        return $this->hasOne('\App\Models\Online', 'user_id');
    }

    public function isNewUser()
    {
        return (bool) $this->newUser;
    }

    public function packages()
    {
        return $this->hasMany('\App\Models\UsersPackages', 'user_id');
    }

    public function transactions()
    {
        return $this->hasMany('\App\Models\Transactions', 'user_id');
    }

    public function balances()
    {
        return $this->hasOne('\App\Models\UsersBalances', 'users_id');
    }

    public function processors()
    {
        return $this->hasMany('\App\Models\UsersProcessors', 'user_id', 'id');
    }

    public function deposits(){
        return $this->hasMany(UsersDeposits::class, 'user_id', 'id');
    }

    public function hasBids() {
        return $this->hasOne('\App\Models\AuctionUsersPackages', 'user_id', 'id')->sum('bids');
    }

    public function deduceBid($item_price_per_bid = 0.01) {

        $act_bid_value = config('auction.bid_price');

        $number = $item_price_per_bid / $act_bid_value;

        $query = \App\Models\AuctionUsersPackages::with('package')->where('user_id', $this->id)->orderBy('created_at', 'ASC')->get();
        $used = [];
        $packages = [];

        foreach( $query as $q ) {
            $used[ ] = [
                'number' => $number,
                'price' => $q->package->bidsPrice()
            ];

            if( $q->bids < $number ) {
                $number = $number - $q->bids;
                $q->bids = 0;
            } else {
                $q->bids = $q->bids - $number;
                $number = 0;
            }

            $packages[] = $q;

            if( $number == 0 )
                break;
        }

        if($number == 0){
            foreach($packages as $q){
                $q->save();

                if( $q->bids == 0 ) {
                    NotificationManager::create([
                        'user_id' => $this->id,
                        'type' => 'user-auction-package-runout',
                        'content_id' => $q->package_id
                    ], $this);

                    $q->delete();
                }
            }
        }else{
            return false;
        }

        return $used;
    }

    public function isSuspended()
    {
        return $this->suspended;
    }

    public function isFrozen()
    {
        return $this->frozen;
    }

    public function isActivated()
    {
        return (bool) $this->activated_at;
    }

    public function isVerified()
    {
        return (bool) $this->verified_at;
    }

    public function getActivationCode()
    {
        $this->activation_code = $activationCode = str_random(42);
        $this->save();

        return $activationCode;
    }

    public function attemptActivation( $activationCode )
    {
        if ( $this->isActivated() ) return false;

        if ( $activationCode == $this->activation_code )
        {
            $this->activation_code = null;
            $this->activated_at    = Carbon::now();
            $this->activated       = 1;
            return $this->save();
        }

        return false;
    }

    public function recordLogin(Request $request)
    {
        $this->last_login = $this->freshTimestamp();
        $this->last_login_ip = $request->getClientIp();
        $this->save();
    }

    public function fullName()
    {
        return $this->first_name . ' ' . $this->last_name;
    }

    public function getAvatar()
    {
        return asset('avatars/avatar.png');
    }

    public function addGroup( $group ) {
        if( 
            !is_null( 
                $role = Role::where('id', $group->id)->first()
            ) 
        ) {
            if( !$this->hasRole( $role->name ) ) {
                $this->assignRole( $role->name );
            }
        }
    }

    public function removeGroup( $group ) {
        if( 
            !is_null( 
                $role = Role::where('id', $group->id)->first()
            ) 
        ) {
            if( $this->hasRole( $role->name ) ) {
                $this->removeRole( $role->name );
            }
        }
    }

    public function kycs_identification() {
        return $this->hasMany('\App\Models\UsersKycs', 'user_id', 'id')->where('type', '=', 'identification')->where('status', '=', null );
    }

    public function kycs_address()
    {
        return $this->hasMany('\App\Models\UsersKycs', 'user_id', 'id')->where('type', '=', 'address')->where('status', '=', null);
    }


    public function sendPasswordResetNotification($token)
    {
        $this->notify(new ResetPasswordNotification($token));
    }

    public function isInTree(){
        return TreeGenealogyNested::where('id', $this->id)->exists();
    }

    public function genealogy()
    {
        return $this->hasOne('App\Models\TreeGenealogyNested', 'id', 'id');
    }

    public function genealogyParent()
    {
        return $this->belongsTo('App\Models\TreeGenealogyNested', 'id', 'parent_id');
    }

    public function getGenealogyParentUser()
    {
        return User::where('id', $this->genealogy()->first()->parent_id)->first();
    }

    public function waitingForMLM(){

        //dd(TempSponsor::where('user_id', $this->id)->exists());

        return TempSponsor::where('user_id', $this->id)->exists();
    }

    public function hasPackage() {
        return !!$this->hasOne('\App\Models\UsersPackages', 'user_id', 'id')->count();
    }

    public function getActivePackage() {
        
        if($this->isSuspended()){
            return null;
        }

        return $this->hasOne('\App\Models\UsersPackages', 'user_id', 'id')->where('status', 1)->first();
    }


    public function getUserLanguage(){
        $language = 'en';

        if(!is_null($profile = $this->profile()->first())){
            $country = $profile->country()->first();

            if($country){
                if($country->code == 'CN'){
                    return strtolower($country->code);
                }
            }
        }

        return $language;
    }
}
